/* tinit.c -- Test mpfi_init.

Copyright 2010,
                     Spaces project, Inria Lorraine
                     and Salsa project, INRIA Rocquencourt,
                     and Arenaire project, Inria Rhone-Alpes, France
                     and Lab. ANO, USTL (Univ. of Lille),  France


This file is part of the MPFI Library.

The MPFI Library is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or (at your
option) any later version.

The MPFI Library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
License for more details.

You should have received a copy of the GNU Lesser General Public License
along with the MPFI Library; see the file COPYING.LIB.  If not, write to
the Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
MA 02110-1301, USA. */

#include "mpfi-tests.h"

int
main (int argc, char **argv)
{
  mpfi_t a;

  mpfi_init (a);
  if (mpfi_get_prec (a) != mpfr_get_default_prec ()) {
    printf ("Error: mpfi_init does not use the default precision of MPFR."
            "\nDefault precision in MPFI: %lu"
            "\nDefault precision in MPFR: %lu\n",
            mpfi_get_prec (a), mpfr_get_default_prec ());

    exit (1);
  }
  mpfi_clear (a);

  mpfr_set_default_prec (111);
  mpfi_init (a);
  if (mpfi_get_prec (a) != mpfr_get_default_prec ()) {
    printf ("Error: mpfi_init does not use the default precision of MPFR."
            "\nDefault precision in MPFI: %lu"
            "\nDefault precision in MPFR: %lu\n",
            mpfi_get_prec (a), mpfr_get_default_prec ());

    exit (1);
  }
  mpfi_clear (a);

  return 0;
}
